#ifndef CORA_FIT_H
#define CORA_FIT_H

#define C_LIGHT 299792.458
#define LIKE_PENALTY 10000.000

#define MAXPATH 512
#define MAXTEXTL 128
#define NENTRY   23
#define MAXLINES 128
#ifndef TRUE
#define TRUE 1
#endif
#ifndef FALSE
#define FALSE 0
#endif

#include "config.h"

typedef struct v_type {
        double * vec;
        int      len;
} Vector;


extern char datafile[MAXPATH];
extern char conffile[MAXPATH];
extern char proffile[MAXPATH];
extern double dt;

/*     vectors read from the spectrum           */
extern Vector wave;	/* wavelength vector	*/
extern Vector sp;	/* count vector		*/
extern Vector err;	/* count errors		*/
extern Vector bkgd;	/* instr. background	*/
extern Vector bkerr;	/* error in background	*/
extern Vector bg0;	/* original background	*/
extern double exposure;	/* exposure time from datafile */

/*   vectors restricted to the selected part of the spectrum  */
extern Vector good;	/* indices of considered wavelength interval */
extern int    count;    /* number of indices in good  */
extern double *yc[MAXLINES];	/* matrix with nlines */
extern Vector x;
extern Vector ncount;
extern Vector dx;
extern Vector bbb;
extern Vector bb0;
extern double binsize;

/*   fit results  */
extern Vector llambda;	/* lambda		*/
extern Vector erlam;		/* error for lambda	*/
extern Vector lsigma; 	/* sigma		*/
extern Vector ersig;		/* error for sigma	*/
extern Vector ampout; 	/* amplitude		*/
extern Vector eramp;		/* error for amplitude	*/ 
extern Vector steplam;	/* line distances to be kept constant */
extern Vector stepsig;        /* distance between sigmas */
extern Vector f;	/* array with weight factors for auxiliary conditions
			 * in the form sum(f_i*ampout_i)=v */
extern double v;

extern Vector lP;            /* lower borders for all variables        */
extern Vector mP;            /* start vector for iteration with Powell */
extern Vector uP;            /* upper borders for all variables        */
extern Vector startP;        /* start vector for all variables         */
extern double (*funcname)();  /* name of functions used by Powell */
extern void (*proffunc)();  /* name of profile functions used in like.c */

/*  input parameters */
extern char fittitle[MAXTEXTL];
extern double lamstart;	  /* lower bound of investigated wavelength interval*/
extern double lamend;	  /* upper bound of investigated wavelength interval*/
extern double dlam;       /* search interval for lambda */
extern int    nlines;	  /* number of lines   */
extern int    idlam;	  /* switch for constant distance of lines 
				(-> steplam=const) */
extern int    idsig;      /* switch for same sigma for all lines */
extern double app_s;	  /* apperture of source image */
extern double app_b;	  /* apperture of backgr image */
extern int fixflag;       /* switch decides whether badly binned spectra will be fixed ( empty bins filles with zeros) */
/*  extern int    num_sig;	   parameter for width of interval within which
				likelihood is determined (cf. fname3) */
extern double lsig;	  /* lower value for sigma */
extern double usig;	  /* upper value for sigma */
extern double dsig;       /* search interval for sigma */
extern int    sigunit;	  /* units of sigma: =1 -> km/s else Angstroem */
extern int    ikev;	  /* unit of wavelength in plot: =1 in keV else A */
extern char   *linID;	  /* label of line */
extern int    ilamit;	  /* switch for iteration of lambda */
extern int    isigit;	  /* switch for iteration of sigma */
extern int    sbgit;	  /* switch for iteration of source background */
extern int    bgsmooth;	  /* switch for smoothing of instrumental backgr */
extern int    sbgchi;     /* switch for iteration of sbg with chi^2 */
extern double medsig;     /* allowed interval for median background */
extern int    bg_dg;      /* degree of polylom for background smoothing  */
extern int    iformat;	  /* switch for specifying the output format  */
extern int    iportrait;  /* switch for orientation (=1 is portrait)  */
extern int    ires;	  /* =1 print results into file 'cora.dat' */
extern int    iplot;	  /* =1 create plot of fit */
extern int    spec_only;  /* =1 plot only the spectrum; no fitting */
extern int    fit_only;  /* =1 plot only the fit, no spectrum */
extern int    ilab;	  /* labels will be insterted into plot
				at adequate positions */
extern int    ierrplot;	  /* switch for drawing error bars in plot */
extern int    ilog;	  /* switch for logarithmic plot */
extern int    nbin;	  /* number of bins beeing combined */
extern int    icount;	  /* =1: counts will be conserved 
			     =0: flux will be conserved */
extern int    igraph;     /* >0 lines can be selected from spectrum
                               (calling option -g) */
extern double iord;	  /* factor to shift multiplet, e.g. for higher order*/
extern int    nlik;	  /* switch for plotting liklihood curves
				(also number of bins) */
extern int    ispline;	  /* switch for fitting liklihood curves
				with cubic splines */
extern int    iuse_fixpt[MAXLINES]; /* switch to use fixed point equation */

extern double sbg;

extern int    verbose;    /* controls verbosity */



#define P_MIN    0
#define P_GAUSS  0
#define P_LORENZ 1
#define P_MAX    1

extern int    iprof;          /* flag for eventual numerical profile =1   */ 
extern int    pro_type[MAXLINES];            /* the profile type          */
extern float  eta_voig[MAXLINES];            /* the mixing parameter      */
extern int    fit_wave[MAXLINES];            /* whether to fit wavelength */
extern int    fit_sigm[MAXLINES];            /* whether to fit sigme      */
extern int    fit_lock[MAXLINES];            /* lock lambda to previous   */
extern float  beta[MAXLINES];		     /* exponent in Beta model    */
extern int    imaxfit;                       /* currently unused, but set */
extern int    iget_final_result;             /* TRUE: switch off Voigt    */

/*  ------------------------ */
/*   sub procedures  */
int read_input (char *path, int s_flag);
int replot(char *path, char *plotfile);
int read_spec (char *path, int s_flag);
int check_bins();
void graph();
void lamran();

double sbgfunc(double * p_vec, int * N);
double sbgiterate(Vector *sarr, double sbgmed);
void smoothbg();
void Amperr(Vector *amp);
void para_err();
void checkerr();

/*   sub functions  */ 
int gaussint (Vector * inp, Vector * out);
double likelihood();
void profile();
void num_profile();
double callik();
void iteration();
void borders();
int penalty(Vector *P, Vector *minP, Vector *maxP);
double func_lik(double * p_vec, int *N);
double func_clik(double *p_vec, int *N);
double func_lamclik(double *p_vec, int *N);
double func_lamlik(double *p_vec, int *N);
double func_clamlik(double *p_vec, int *N);
double func_siglik(double *p_vec, int *N);
double func_sigclik(double *p_vec, int *N);
double func_csiglik(double *p_vec, int *N);
double func_testlik(Vector *P);
double gcurve(double x, double lam, double sig, double dx);
double errlik(double x);

/*  dealing with vectors  */
void vec_init(Vector *x);
void vec_alloc(Vector *x, int n);
void vec_free(Vector *x);

/*  useful function  */
double max(Vector *x);
double min(Vector *x);
double median(Vector *x);
double total(Vector *x);
double p_fit(Vector *c, Vector *n);
int test_confdir ();
char * get_home_dir();
void print_vec(Vector *amp);

int poly_fit (Vector * x, Vector * y, Vector * bgs, int m);
int powell (double * startvec, int n, double ftol, double * fmin,
            double (*f)(), double dstep);
/* startvec: starting point (n-dimensional),
 *           on output replaced with the location of the minimum.
 * n       : dimension of startvec
 * ftol    : tolerance (convergence criterium)
 * fmin    : contains function value on output
 * f       : a function defined as:   double function_name(double * x, int n)
 */
int MinFinder (double * t2, double low, double up,
               double ftol, double * fmin, double (*func)());

double zeroin (double ax, double bx,
                   double (*f)(double x), double tol);

/*   stuff concerning where function  */
void where(Vector * ret, Vector * x, int (*func)(), double *a, int * count);
void where2(Vector * ret, Vector * x, int (*func)(), double *a, double *b, int * count);
int func_gt(double *a, double *b);
int func_lt(double *a, double *b);
int func_ge(double *a, double *b);
int func_ul(double *a, double *b, double *c);
void sel_vals(Vector *ret, Vector *x, Vector *good);

int plotfit (char * fname       /* title             */,
	     char * psfile      /* filename if device PS, GIF or PPM */,
	     int    fileformat  /* XSERVE/PS/GIF/PPM */
	     );
void plotlik_lam(int index);
void plotlik_sig(int index);
void plotlik_lam_all(double (*func)(), double * p_vec, int * N, int clflag);
void note_lik(Vector * P, double like, int clflag);

#endif
